@@ -1,58 +1,50 @@
-use crate::components::{iter_bit_indexes, Component, ComponentStorages, FamilyMask, GroupMask};
-use crate::storage::{ComponentStorage, Entity, EntityStorage};
-use crate::utils::{panic_missing_comp, ChangeTicks};
-use atomic_refcell::AtomicRefMut;
+use crate::components::{ComponentStorages, FamilyMask, GroupMask};
+use crate::storage::{Component, ComponentStorage, Entity, EntityStorage};
+use crate::utils::panic_missing_comp;
 use std::any::TypeId;
+use std::iter;
 
-/// Handles adding/removing `Component`s to/from storages.
-/// Trait implemented by `Component` tuples up to arity 16.
+/// Manages components in component storages.
+/// Implemented for `Component` tuples up to arity 16.
 ///
 /// # Safety
 /// All operations must preserve component grouping.
-pub unsafe trait ComponentSet
-where
-    Self: Sized + Send + Sync + 'static,
-{
-    /// Inserts the `entity` and its `components` into the `storages`.
-    fn insert(
-        storages: &mut ComponentStorages,
-        entity: Entity,
-        components: Self,
-        ticks: ChangeTicks,
-    );
+pub unsafe trait ComponentSet: Sized + Send + Sync + 'static {
+    /// Result of the `remove` operation.
+    type RemoveResult: Send + Sync + 'static;
 
-    /// Creates new entities with `Component`s produced by `components_iter`.
-    /// Returns the newly created entities as a slice.
+    /// Adds the given `components` to `entity`.
+    fn insert(storages: &mut ComponentStorages, entity: Entity, components: Self);
+
+    /// Creates new entities with the components produced by the iterator.
     fn extend<'a, I>(
         entities: &'a mut EntityStorage,
         storages: &mut ComponentStorages,
         components_iter: I,
-        ticks: ChangeTicks,
     ) -> &'a [Entity]
     where
         I: IntoIterator<Item = Self>;
 
-    /// Removes the `entity` and its `Component`s from the `storages` and
-    /// returns the `Component`s if they were successfully removed.
+    /// Removes a component set from `entity` and returns the components.
     #[must_use = "use `delete` to discard the components"]
-    fn remove(storages: &mut ComponentStorages, entity: Entity) -> Option<Self>;
+    fn remove(storages: &mut ComponentStorages, entity: Entity) -> Self::RemoveResult;
 
-    /// Deletes the `entity` and its `Component`s from the `storages`. This is
-    /// faster than removing them.
+    /// Deletes a component set from `entity`. This is faster than removing them.
     fn delete(storages: &mut ComponentStorages, entity: Entity);
 }
 
 unsafe impl ComponentSet for () {
+    type RemoveResult = ();
+
     #[inline]
-    fn insert(_: &mut ComponentStorages, _: Entity, _: Self, _: ChangeTicks) {
+    fn insert(_storages: &mut ComponentStorages, _entity: Entity, _components: Self) {
         // Empty
     }
 
     fn extend<'a, I>(
         entities: &'a mut EntityStorage,
-        _: &mut ComponentStorages,
+        _storages: &mut ComponentStorages,
         components_iter: I,
-        _: ChangeTicks,
     ) -> &'a [Entity]
     where
         I: IntoIterator<Item = Self>,
@@ -65,12 +57,12 @@ unsafe impl ComponentSet for () {
     }
 
     #[inline]
-    fn remove(_: &mut ComponentStorages, _: Entity) -> Option<Self> {
-        Some(())
+    fn remove(_storages: &mut ComponentStorages, _entity: Entity) -> Self::RemoveResult {
+        ()
     }
 
     #[inline]
-    fn delete(_: &mut ComponentStorages, _: Entity) {
+    fn delete(_storages: &mut ComponentStorages, _entity: Entity) {
         // Empty
     }
 }
@@ -81,27 +73,26 @@ macro_rules! impl_component_set {
         where
             $($comp: Component,)+
         {
+            type RemoveResult = ($(Option<$comp>,)+);
+
             #[allow(clippy::eval_order_dependence)]
             fn insert(
                 storages: &mut ComponentStorages,
                 entity: Entity,
                 components: Self,
-                ticks: ChangeTicks,
             ) {
                 let mut family_mask = FamilyMask::default();
 
                 $(
                     {
-                        let (storage, mask) = get_with_family_mask_mut::<$comp>(storages);
-                        unsafe { storage.insert::<$comp>(entity, components.$idx, ticks); }
+                        let (storage, mask) = get_as_ptr_with_family_mask::<$comp>(storages);
+                        unsafe { (*storage).insert::<$comp>(entity, components.$idx); }
                         family_mask |= mask;
                     }
                 )+
 
-                for i in iter_bit_indexes(family_mask) {
-                    unsafe {
-                        storages.group_components(i, Some(&entity));
-                    }
+                unsafe {
+                    storages.group_families(family_mask, iter::once(entity));
                 }
             }
 
@@ -110,7 +101,6 @@ macro_rules! impl_component_set {
                 entities: &'a mut EntityStorage,
                 storages: &mut ComponentStorages,
                 components_iter: It,
-                ticks: ChangeTicks,
             ) -> &'a [Entity]
             where
                 It: IntoIterator<Item = Self>
@@ -119,9 +109,9 @@ macro_rules! impl_component_set {
                 let mut family_mask = FamilyMask::default();
 
                 {
-                    let mut borrowed_storages = (
+                    let storage_ptrs = (
                         $({
-                            let (storage, mask) = borrow_with_family_mask_mut::<$comp>(storages);
+                            let (storage, mask) = get_as_ptr_with_family_mask::<$comp>(storages);
                             family_mask |= mask;
                             storage
                         },)+
@@ -131,52 +121,40 @@ macro_rules! impl_component_set {
                         let entity = entities.create();
 
                         unsafe {
-                            $(borrowed_storages.$idx.insert::<$comp>(entity, components.$idx, ticks);)+
+                            $(
+                                (*storage_ptrs.$idx).insert::<$comp>(entity, components.$idx);
+                            )+
                         }
                     });
                 }
 
-                let new_entities = unsafe { entities.get_unchecked(initial_entity_count..) };
-
-                for i in iter_bit_indexes(family_mask) {
-                    unsafe {
-                        storages.group_components(i, new_entities);
-                    }
+                unsafe {
+                    let new_entities = entities.get_unchecked(initial_entity_count..);
+                    storages.group_families(family_mask, new_entities.iter().copied());
+                    new_entities
                 }
-
-                new_entities
             }
 
             #[allow(clippy::eval_order_dependence)]
-            fn remove(storages: &mut ComponentStorages, entity: Entity) -> Option<Self> {
+            fn remove(storages: &mut ComponentStorages, entity: Entity) -> Self::RemoveResult {
                 let mut family_mask = FamilyMask::default();
                 let mut group_mask = GroupMask::default();
 
                 let storage_ptrs = ($(
                     {
-                        let (storage_ptr, family, group) = storages.get_as_ptr_with_masks(&TypeId::of::<$comp>());
-
-                        if storage_ptr.is_null() {
-                            panic_missing_comp::<$comp>();
-                        }
+                        let (storage, family, group) = get_as_ptr_with_masks::<$comp>(storages);
 
                         family_mask |= family;
                         group_mask |= group;
-                        storage_ptr
+
+                        storage
                     },
                 )+);
 
-                for i in iter_bit_indexes(family_mask) {
-                    unsafe {
-                        storages.ungroup_components(i, group_mask, Some(&entity));
-                    }
+                unsafe {
+                    storages.ungroup_families(family_mask, group_mask, iter::once(entity));
+                    ($((*storage_ptrs.$idx).remove::<$comp>(entity),)+)
                 }
-
-                let components = unsafe { (
-                    $((&mut *storage_ptrs.$idx).remove::<$comp>(entity),)+
-                ) };
-
-                Some(($(components.$idx?,)+))
             }
 
             #[allow(clippy::eval_order_dependence)]
@@ -186,52 +164,48 @@ macro_rules! impl_component_set {
 
                 let storage_ptrs = ($(
                     {
-                        let (storage_ptr, family, group) = storages.get_as_ptr_with_masks(&TypeId::of::<$comp>());
-
-                        if storage_ptr.is_null() {
-                            panic_missing_comp::<$comp>();
-                        }
+                        let (storage, family, group) = get_as_ptr_with_masks::<$comp>(storages);
 
                         family_mask |= family;
                         group_mask |= group;
-                        storage_ptr
+
+                        storage
                     },
                 )+);
 
-                for i in iter_bit_indexes(family_mask) {
-                    unsafe {
-                        storages.ungroup_components(i, group_mask, Some(&entity));
-                    }
-                }
-
                 unsafe {
-                    $((&mut *storage_ptrs.$idx).remove::<$comp>(entity);)+
+                    storages.ungroup_families(family_mask, group_mask, iter::once(entity));
+                    $((*storage_ptrs.$idx).delete_typed::<$comp>(entity);)+
                 }
             }
         }
     };
 }
 
-fn get_with_family_mask_mut<T>(
-    storages: &mut ComponentStorages,
-) -> (&mut ComponentStorage, FamilyMask)
+fn get_as_ptr_with_family_mask<T>(
+    storages: &ComponentStorages,
+) -> (*mut ComponentStorage, FamilyMask)
 where
     T: Component,
 {
-    storages
-        .get_with_family_mask_mut(&TypeId::of::<T>())
-        .unwrap_or_else(|| panic_missing_comp::<T>())
+    let (storage, family_mask) = storages
+        .get_as_ptr_with_family_mask(&TypeId::of::<T>())
+        .unwrap_or_else(|| panic_missing_comp::<T>());
+
+    (storage.as_ptr(), family_mask)
 }
 
-fn borrow_with_family_mask_mut<T>(
+fn get_as_ptr_with_masks<T>(
     storages: &ComponentStorages,
-) -> (AtomicRefMut<ComponentStorage>, FamilyMask)
+) -> (*mut ComponentStorage, FamilyMask, GroupMask)
 where
     T: Component,
 {
-    storages
-        .borrow_with_family_mask_mut(&TypeId::of::<T>())
-        .unwrap_or_else(|| panic_missing_comp::<T>())
+    let (storage, family_mask, group_mask) = storages
+        .get_as_ptr_with_masks(&TypeId::of::<T>())
+        .unwrap_or_else(|| panic_missing_comp::<T>());
+
+    (storage.as_ptr(), family_mask, group_mask)
 }
 
 #[rustfmt::skip]
