@@ -1,11 +1,7 @@
 use downcast_rs::{impl_downcast, Downcast};
 
-/// Trait automatically implemented for all types that can be stored in
-/// `Resources`.
-pub trait Resource
-where
-    Self: Downcast,
-{
+/// Trait automatically implemented for all types that can be used as resources.
+pub trait Resource: Downcast {
     // Empty
 }
 
