@@ -1,8 +1,8 @@
 use std::fmt;
 use std::num::NonZeroU32;
 
-/// Used by `EntityStorage` to recycle indexes. Entities with the same id and
-/// different `Version`s are considered different.
+/// Type used to tell apart entities with the same id. Entities with the same id and different
+/// `Version`s are considered different.
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug)]
 pub struct Version(NonZeroU32);
 
@@ -30,7 +30,7 @@ impl Version {
     }
 }
 
-/// Handle used to fetch components from `ComponentStorages`.
+/// Uniquely identifies a set of components in a `World`.
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub struct Entity {
     id: u32,
@@ -39,27 +39,21 @@ pub struct Entity {
 
 impl fmt::Debug for Entity {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("Entity")
-            .field("id", &self.id)
-            .field("version", &self.version.0)
-            .finish()
+        f.debug_struct("Entity").field("id", &self.id).field("version", &self.version.0).finish()
     }
 }
 
 impl Entity {
-    /// Creates a new entity with the given id`and `Version`.
+    /// Creates a new entity with the given `id` and `Version`.
     #[inline]
     pub const fn new(id: u32, version: Version) -> Self {
         Self { id, version }
     }
 
-    /// Creates a new entity with the given id and default `Version`.
+    /// Creates a new entity with the given `id` and default `Version`.
     #[inline]
     pub const fn with_id(id: u32) -> Self {
-        Self {
-            id,
-            version: Version::DEFAULT,
-        }
+        Self { id, version: Version::DEFAULT }
     }
 
     /// Returns the id of the entity.
@@ -68,8 +62,7 @@ impl Entity {
         self.id
     }
 
-    /// Returns the id of the entity extended to a usize. Used as an index into
-    /// `EntitySparseArray`s.
+    /// Returns the id of the entity extended to a usize.
     #[inline]
     pub const fn sparse(&self) -> usize {
         self.id as _
@@ -82,10 +75,10 @@ impl Entity {
     }
 }
 
-/// Used internally by `EntitySparseArray` to map `Entity` indexes to dense
+/// Used internally by `SparseArray` to map `Entity` indexes to dense
 /// indexes.
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
-pub struct IndexEntity {
+pub(crate) struct IndexEntity {
     id: u32,
     version: Version,
 }
@@ -106,8 +99,7 @@ impl IndexEntity {
         Self { id, version }
     }
 
-    /// Returns the `id` of the entity extended to a `usize`. Used to index
-    /// dense arrays.
+    /// Returns the `id` of the entity extended to a `usize`..
     #[inline]
     pub const fn dense(&self) -> usize {
         self.id as _
