@@ -1,44 +1,78 @@
 use crate::storage::{Entity, IndexEntity};
-use crate::utils::UnsafeUnwrap;
 use std::{iter, ptr};
 
-const PAGE_SIZE: usize = 32;
+const PAGE_SIZE: usize = 64;
 type EntityPage = Option<Box<[Option<IndexEntity>; PAGE_SIZE]>>;
 
+/// Maps sparse indexes to dense indexes. Used internally by `ComponentStorage`.
+#[doc(hidden)]
 #[derive(Clone, Debug, Default)]
-pub(crate) struct SparseArray {
+pub struct SparseArray {
     pages: Vec<EntityPage>,
 }
 
 impl SparseArray {
-    /// Returns whether or not the array contains `entity`.
-    pub fn contains(&self, entity: Entity) -> bool {
-        self.get_index(entity).is_some()
-    }
-
-    /// Returns the index mapped to `entity`.
-    pub fn get_index(&self, entity: Entity) -> Option<usize> {
+    /// Returns the dense index mapped to `entity`, if any.
+    pub fn get(&self, entity: Entity) -> Option<usize> {
         self.pages
             .get(page_index(entity))
-            .and_then(|p| p.as_ref())
-            .and_then(|p| p[local_index(entity)])
+            .and_then(Option::as_ref)
+            .and_then(|p| p[local_index(entity)].as_ref())
             .filter(|e| e.version() == entity.version())
-            .map(|e| e.index())
+            .map(IndexEntity::dense)
+    }
+
+    /// Returns `true` if the array contains `entity`.
+    pub fn contains(&self, entity: Entity) -> bool {
+        self.pages
+            .get(page_index(entity))
+            .and_then(Option::as_ref)
+            .and_then(|p| p[local_index(entity)].as_ref())
+            .filter(|e| e.version() == entity.version())
+            .is_some()
+    }
+
+    /// Returns the dense index mapped to `sparse`, if any.
+    pub fn get_from_sparse(&self, sparse: usize) -> Option<usize> {
+        self.pages
+            .get(sparse / PAGE_SIZE)
+            .and_then(Option::as_ref)
+            .and_then(|p| p[sparse % PAGE_SIZE].as_ref())
+            .map(IndexEntity::dense)
+    }
+
+    /// Returns `true` if the array contains `sparse`.
+    pub fn contains_sparse(&self, sparse: usize) -> bool {
+        self.pages
+            .get(sparse / PAGE_SIZE)
+            .and_then(Option::as_ref)
+            .and_then(|p| p[sparse % PAGE_SIZE].as_ref())
+            .is_some()
+    }
+
+    pub(crate) fn remove(&mut self, entity: Entity) -> Option<usize> {
+        self.pages
+            .get_mut(page_index(entity))
+            .and_then(Option::as_mut)
+            .map(|p| &mut p[local_index(entity)])
+            .filter(|e| e.map(|e| e.version() == entity.version()).unwrap_or(false))?
+            .take()
+            .map(|e| e.dense())
     }
 
     /// Returns the `IndexEntity` slot at `index` without checking if the
     /// `index` it is valid.
-    pub unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut Option<IndexEntity> {
+    pub(crate) unsafe fn get_unchecked_mut(&mut self, index: usize) -> &mut Option<IndexEntity> {
         self.pages
             .get_unchecked_mut(index / PAGE_SIZE)
             .as_mut()
-            .unsafe_unwrap()
+            .unwrap_unchecked()
             .get_unchecked_mut(index % PAGE_SIZE)
     }
 
     /// Returns the `IndexEntity` slot at `index`. May allocate memory if the
     /// index cannot be stored in the allocated pages.
-    pub fn get_mut_or_allocate_at(&mut self, index: usize) -> &mut Option<IndexEntity> {
+    pub(crate) fn get_mut_or_allocate_at(&mut self, index: usize) -> &mut Option<IndexEntity> {
         let page_index = index / PAGE_SIZE;
 
         if page_index < self.pages.len() {
@@ -50,8 +84,7 @@ impl SparseArray {
         } else {
             let extra_uninit_pages = page_index - self.pages.len();
             self.pages.reserve(extra_uninit_pages + 1);
-            self.pages
-                .extend(iter::repeat(uninit_page()).take(extra_uninit_pages));
+            self.pages.extend(iter::repeat(uninit_page()).take(extra_uninit_pages));
             self.pages.push(empty_page())
         }
 
@@ -59,77 +92,31 @@ impl SparseArray {
             self.pages
                 .get_unchecked_mut(page_index)
                 .as_mut()
-                .unsafe_unwrap()
+                .unwrap_unchecked()
                 .get_unchecked_mut(index % PAGE_SIZE)
         }
     }
 
     /// Swaps the `IndexEntities` at `a` and `b` without checking if `a` and `b`
     /// are valid.
-    pub unsafe fn swap_unchecked(&mut self, a: usize, b: usize) {
+    pub(crate) unsafe fn swap_nonoverlapping(&mut self, a: usize, b: usize) {
         let pa = self.get_unchecked_mut(a) as *mut _;
         let pb = self.get_unchecked_mut(b) as *mut _;
-        ptr::swap(pa, pb);
-    }
-
-    /// Removes `entity` from the array and returns the `index` mapped to it.
-    pub fn remove(&mut self, entity: Entity) -> Option<usize> {
-        Some(self.get_mut(entity)?.take()?.index())
+        ptr::swap_nonoverlapping(pa, pb, 1);
     }
 
     /// Removes all entities from the array.
-    pub fn clear(&mut self) {
+    pub(crate) fn clear(&mut self) {
         self.pages.iter_mut().for_each(|p| *p = None);
     }
-
-    /// Returns a read-only view of the array.
-    pub fn as_view(&self) -> SparseArrayView {
-        SparseArrayView { pages: &self.pages }
-    }
-
-    fn get_mut(&mut self, entity: Entity) -> Option<&mut Option<IndexEntity>> {
-        self.pages
-            .get_mut(page_index(entity))
-            .and_then(|page| page.as_mut())
-            .map(|page| &mut page[local_index(entity)])
-            .filter(|e| e.map(|e| e.version()) == Some(entity.version()))
-    }
-}
-
-/// Shared view over a `SparseArray`.
-#[derive(Copy, Clone)]
-pub struct SparseArrayView<'a> {
-    pages: &'a [EntityPage],
-}
-
-impl SparseArrayView<'_> {
-    /// Returns `true` if the view contains `entity`.
-    pub fn contains(&self, entity: Entity) -> bool {
-        self.pages
-            .get(page_index(entity))
-            .and_then(|p| p.as_ref())
-            .and_then(|p| p[local_index(entity)])
-            .filter(|e| e.version() == entity.version())
-            .is_some()
-    }
-
-    /// Returns the index mapped to `entity` if it exists.
-    pub fn get_index(&self, entity: Entity) -> Option<usize> {
-        self.pages
-            .get(page_index(entity))
-            .and_then(|p| p.as_ref())
-            .and_then(|p| p[local_index(entity)])
-            .filter(|e| e.version() == entity.version())
-            .map(|e| e.index())
-    }
 }
 
 fn page_index(entity: Entity) -> usize {
-    entity.index() / PAGE_SIZE
+    entity.sparse() / PAGE_SIZE
 }
 
 fn local_index(entity: Entity) -> usize {
-    entity.index() % PAGE_SIZE
+    entity.sparse() % PAGE_SIZE
 }
 
 fn uninit_page() -> EntityPage {
